library(ggplot2)
library(sqldf)
library(plyr)
library(cowplot)

thunder_ms <- read.csv("thunder_configure_15.csv")

thunder_ms <- sqldf("select run,ident,jmax,links,latency, CAST(latency as real) / 1000. as lat_ms from thunder_ms")
thunder_ms$links <- as.factor(thunder_ms$links)
thunder_ms$jmax <- as.factor(thunder_ms$jmax)

v1 <- ggplot(data = thunder_ms, aes(x = jmax, y=lat_ms, fill=links)) +
  geom_violin() +
  geom_hline(yintercept=800, color = "red", linetype="dashed") +
  geom_hline(yintercept=400, color = "DarkGreen") +
  annotate("text", x=6, y=550, label= "high", color="DarkGreen") +
  annotate("text", x=5.7, y=1000, label= "acceptable", color="red") +
  coord_cartesian(xlim = c(1, 6)) +
  scale_fill_grey() +
  #scale_y_log10() +
  ylab("latency (ms)") +
  xlab("max allowed jitter") +
  theme_classic()

#v1 + ggsave("thunder_configure_ms.png", dpi=300, dev='png', height=5, width=15, units="cm")

thunder_rcv <- sqldf("select run,jmax,links,(9900-COUNT(latency)*1.0)/9900 as dlv from thunder_ms group by jmax,links,run")
thunder_rcv$jmax <- as.factor(thunder_rcv$jmax)
thunder_rcv$links <- as.factor(thunder_rcv$links)
v2 <- ggplot(data = thunder_rcv, aes(x = jmax, y=dlv, fill=links)) +
  geom_boxplot() +
  scale_y_continuous(labels = scales::percent) +
  scale_fill_grey() +
  ylab("dropped packets") +
  xlab("max allowed jitter") +
  theme_classic()

thunder_bw <- read.csv("thunder_configure_15_bw.csv")
thunder_bw <- sqldf("select run,jmax,links,udp_sent,udp_rcv,cells_sent,cells_rcv,1.0*cells_sent/udp_sent as sent_ratio,1.0*cells_rcv/udp_rcv as rcv_ratio from thunder_bw where udp_sent > 4000")
thunder_bw$jmax <- as.factor(thunder_bw$jmax)
thunder_bw$links <- as.factor(thunder_bw$links)

v3 <- ggplot(data = thunder_bw, aes(x = jmax, y=sent_ratio, fill=links)) +
  geom_boxplot() +
  #scale_y_log10() +
  #scale_y_log10(labels = scales::percent) +
  scale_fill_grey() +
  ylab("bandwidth ratio") +
  xlab("max allowed jitter") +
  theme_classic()


t1 <- plot_grid(v1, v2, v3, labels = c('A', 'B', 'C'), ncol=1)
t1 + ggsave("thunder_configure.png", dpi=300, dev='png', height=15, width=15, units="cm")

thunder_links <- read.csv("thunder_configure_15_links.csv")
links_down_at_least_once <- sqldf("select run,xp_time,link_id, COUNT(status) as downcount, SUM(delta) as elapsed from thunder_links where status='down' group by run,link_id,xp_time")
links_down_at_least_once2 <- sqldf("select row_number () OVER (PARTITION BY run ORDER BY elapsed DESC) sorting,run,link_id,downcount,elapsed,xp_time,1.0*elapsed/xp_time down_ratio from links_down_at_least_once")
links_down_at_least_once2$sorting <- as.factor(links_down_at_least_once2$sorting)
v4 <- ggplot(data = links_down_at_least_once2, aes(x = sorting, y=down_ratio)) +
  #geom_violin() +
  #geom_boxplot(width=0.2) +
  #scale_y_log10() +
  #scale_y_log10(labels = scales::percent) +
  geom_bar(stat="summary") +
  #scale_y_log10() +
  scale_y_continuous(labels = scales::percent) +
  scale_fill_grey() +
  ylab("Cumulated downtime") +
  xlab("Sorted links") +
  theme_classic()

downtime <- sqldf(
"
select sorting,1.0*duration/1000 as dur 
from thunder_links as tl
inner join links_down_at_least_once2 as l2 on 
  tl.run = l2.run
  and tl.link_id = l2.link_id 
where will_change='True' 
  and status='down'")

v5 <- ggplot(data = downtime, aes(x=sorting, y=dur)) +
  geom_violin() +
  scale_y_log10() +
  ylab("Downtime (in sec)") +
  xlab("Sorted links") +
  theme_classic()

gobal_links <- sqldf(
"
select ts,run,durations_global,8-COUNT(status) as down_link_count,xp_time
from thunder_links 
where will_change_global='True' and status='up' and durations_global > 0
group by ts,run,durations_global
")

down_group_ratio <- sqldf(
"
select run,1.0 * SUM(durations_global)/xp_time as down_ratio, down_link_count
from gobal_links
where down_link_count >= 0
group by run,down_link_count,xp_time
")

down_group_ratio$down_link_count <- as.factor(down_group_ratio$down_link_count)
v6 <- ggplot(data = down_group_ratio, aes(x=down_link_count, y=down_ratio)) +
  geom_bar(stat="summary") +
  #scale_y_log10() +
  scale_y_continuous(labels = scales::percent) +
  ylab("Cumulated downtime") +
  xlab("Number of links down at once") +
  theme_classic()

downtime_group <- sqldf(
"
select down_link_count, 1.0*durations_global/1000 as dur from gobal_links  where down_link_count >= 0
")
downtime_group$down_link_count <- as.factor(downtime_group$down_link_count)
v7 <- ggplot(data = downtime_group, aes(x=down_link_count, y=dur)) +
  geom_violin() +
  scale_y_log10() +
  ylab("Downtime (in sec)") +
  xlab("Number of links down at once") +
  theme_classic()

t2 <- plot_grid(v4, v5, v6, v7, labels = c('A', 'B', 'C', 'D'), ncol=2)
t2 + ggsave("thunder_links.png", dpi=300, dev='png', height=12, width=15, units="cm")

latency_evol <- sqldf(
"
select sorting,lat_ms,ident,tm.jmax,tm.links
from 
  thunder_ms as tm,
  (select run,jmax,links,row_number () OVER (ORDER BY links DESC) sorting from thunder_ms group by run,jmax,links ORDER BY links DESC  limit 0,1) as sel_run
where
  tm.run = sel_run.run and tm.jmax = sel_run.jmax and tm.links = sel_run.links
")

latency_evol$sorting <- as.factor(latency_evol$sorting)
ggplot(data=latency_evol, aes(x=ident,y=lat_ms)) +
  geom_line() +
  theme_classic()


